from tibian.config import construct_objects_based_on_config_type, load_config
from tibian.tickets import (
    announce_birthdays,
    collect_open_tickets,
    filter_for_birthday_tickets,
)


def main():

    config = load_config()

    sources = construct_objects_based_on_config_type(config["sources"])
    targets = construct_objects_based_on_config_type(config["destinations"])

    open_tickets = collect_open_tickets(sources)
    birthday_tickets = filter_for_birthday_tickets(open_tickets)

    announce_birthdays(targets, birthday_tickets)


if __name__ == "__main__":
    main()



from typing import Any, Dict, List, Sequence, Type, Union

import yaml

import tibian.vars
from tibian.sources import POSSIBLE_SOURCES
from tibian.sources.ticketsource import TicketSource
from tibian.targets import POSSIBLE_DESTINATIONS
from tibian.targets.target import Target

_DEFAULT_CONFIG_PATH = tibian.vars.get_std_config_filepath()


def load_config(filepath=_DEFAULT_CONFIG_PATH) -> Dict:

    with open(filepath) as c:
        config = yaml.safe_load(c)

    return config


def get_cls_from_possible(
    entry_type: str, possible: Sequence[Union[Type[TicketSource], Type[Target]]]
) -> Union[Type[TicketSource], Type[Target]]:

    for pos in possible:
        if pos.TYPENAME == entry_type:
            return pos

    raise ValueError(f"Couldn't find config class for type {entry_type}")


def _get_possible_classes() -> List[Union[Type[TicketSource], Type[Target]]]:

    possible_clss: List[Union[Type[TicketSource], Type[Target]]] = []
    possible_clss.extend(POSSIBLE_SOURCES)
    possible_clss.extend(POSSIBLE_DESTINATIONS)

    return possible_clss


def construct_objects_based_on_config_type(configs: Sequence[Dict[str, Any]]) -> Any:

    possible_clss = _get_possible_classes()

    objects = []
    for entry in configs:
        entry_type = entry["type"]
        cls = get_cls_from_possible(entry_type, possible_clss)
        obj = cls(entry["name"], entry["config"])
        objects.append(obj)

    return objects



import json
from typing import TYPE_CHECKING, List

import requests

if TYPE_CHECKING:
    from tibian.tickets import BirthdayTicket

import tibian.vars
from tibian.targets.target import Target


class TeamsTarget(Target):

    TYPENAME = "teams"

    def __init__(self, name: str, config: dict, *args, **kwargs) -> None:
        super().__init__(name, config, *args, **kwargs)
        self.url = config["url"]

    def announce_birthdays(self, birthday_tickets: List["BirthdayTicket"]) -> None:

        base_content = {
            "@type": "MessageCard",
            "@context": "http://schema.org/extensions",
            "themeColor": "0076D7",
            "summary": "Birthdays",
            "sections": [
                {
                    "activityTitle": f"{tibian.vars.get_today()}",
                    "facts": [],
                    "text": "",
                    "markdown": True,
                }
            ],
        }

        body = base_content["sections"][0]
        assert isinstance(body, dict)

        if birthday_tickets:
            body["facts"] = [
                {"name": ticket.name, "value": f"{ticket.title} ({ticket.age} year(s))"}
                for ticket in birthday_tickets
            ]
        else:
            body["text"] = "**It's no tickets birthday today :(**"

        headers = {"Content-Type": "application/json"}

        res = requests.post(self.url, data=json.dumps(base_content), headers=headers, timeout=10)

        res.raise_for_status()



from typing import TYPE_CHECKING, List

from tibian.targets.target import Target

if TYPE_CHECKING:
    from tibian.tickets import BirthdayTicket


class StdoutTarget(Target):

    TYPENAME = "stdout"

    def __init__(self, name: str, config: dict, *args, **kwargs) -> None:
        super().__init__(name, config, *args, **kwargs)

    def announce_birthdays(self, birthday_tickets: List["BirthdayTicket"]) -> None:

        for bt in birthday_tickets:
            print(f"{bt.name} '{bt.title}': {bt.age}")



import abc
from typing import TYPE_CHECKING, List

if TYPE_CHECKING:
    from tibian.tickets import BirthdayTicket


class Target(abc.ABC):

    TYPENAME = "unknown"

    def __init__(self, name: str, config: dict, *args, **kwargs) -> None:
        super().__init__()
        self.name = name

    @abc.abstractmethod
    def announce_birthdays(self, birthday_tickets: List["BirthdayTicket"]) -> None:
        pass



from typing import List

import dateutil.parser
from jira import JIRA
from jira.client import Issue

from tibian.sources.ticketsource import TicketSource
from tibian.tickets import Ticket


class JiraSource(TicketSource):

    TYPENAME = "jira"

    def __init__(self, name: str, config: dict, *args, **kwargs) -> None:
        super().__init__(name, config)

        url = config["url"]
        auth = config["auth"]
        username, password = auth["username"], auth["password"]

        self.jira = JIRA(url, auth=(username, password))
        self.project = config["project"]

    def get_open_tickets(self) -> List[Ticket]:

        block_size = 250
        block_num = 0
        open_issues = []
        while True:
            start_index = block_num * block_size
            issues = self.jira.search_issues(f"project = {self.project}", start_index, block_size)
            if not issues:
                break
            block_num += 1

            for issue in issues:
                assert isinstance(issue, Issue)
                if issue.fields.status.name not in ["Closed", "Cancelled", "Done"]:
                    open_issues.append(
                        Ticket(
                            name=issue.key,
                            title=issue.fields.summary,
                            status=issue.fields.status.name,
                            creation_date=dateutil.parser.parse(issue.fields.created).date(),
                        )
                    )

        return open_issues



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['tibian', 'tibian.sources', 'tibian.targets']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=6.0,<7.0',
 'jira>=3.1.1,<4.0.0',
 'python-dateutil>=2.8.2,<3.0.0',
 'requests>=2.27.1,<3.0.0',
 'types-PyYAML>=6.0.5,<7.0.0',
 'types-python-dateutil>=2.8.10,<3.0.0',
 'types-requests>=2.27.14,<3.0.0']

entry_points = \
{'console_scripts': ['tibian = tibian.main:main']}

setup_kwargs = {
    'name': 'tibian',
    'version': '1.0.0',
    'description': 'Ticket birthday announcer: A package to announce all creation birthdays of your tickets that live (too) long enough',
    'long_description': "Tibian\n======\n\nProject 'tibian', also known as the 'TIcket Birthday Announcer',\nis a project to announce tickets that it's birthday today.\nThat means it announces ticket titles and summaries that were created a few\nyears ago and are still open to a number of channels that you specified before.\nFor example, the Jira tickets of a project can be published to a Teams channel\nregularly and make some people happy (or angry).\n\nRight now, the project is in a very early stage and it's not fully functional.\n\n\nUsage\n-----\n\nTo use this project, you need to create a configuration file in the same directory.\nTo start, copy the `config.example.yaml` file to `config.yaml` and edit it.\nThe name is important here, because it's used to identify the configuration file.\nFor details about configuration, visit section `Configuration`.\n\nTo install the project, run one of the following command::\n\n    poetry install --no-dev # or\n    pip install tibian\n\nLater one is preferred, as it installs the project with the same versions\nas we tested before release and in a virtual environment.\n\nAfterwards, you can start the project by running the `tibian` command::\n\n    tibian\n\nAfterwards, it should announce the birthdays of tickets of the current date\nas specified.\n\nConfiguration\n-------------\n\nThe configuration file for the project must be located in the root folder\nof the execution command. A short (any maybe not complete) description of\nthe configuration file is given in `config.example.yaml` on GitHub.\n\nCopy this output to a file `config.yaml` in the current directory, add your credentials\nand remove parts you don't need.\n\nDetailed information about all config options will be given soon,\nbut for this version most of the configuration should be self-explanatory.\n\n'type' is the type of source or target you have,\n'name' is an internal used name for the source/target, and\n'config' is a type-dependant dictionary of types as shown.\n\n\nVersioning\n-----------\n\nWe use `semantic versioning`_ to version the project. As we are in some sort of 'beta',\nwe may (but try not to) do some breaking changes between minor versions to fix some obvious\nmisbehavior of the project. For example, we may change the configuration file format or\nadd new options to the configuration file. As we try to prevent this and add a backlog how to\nupgrade to newer versions, you maybe want to check the changelog before updating to new versions.\n\n\nDevelopment\n-----------\n\nWe are always happy about active support. If you want to actively develop on tibian, follow the next few commands.\nWe use poetry_ for the development of tibian. You can install it with the following command::\n\n    pip install poetry\n\nTo install all development dependencies, run::\n\n    poetry install\n\nAfterwards, you have all dependencies (including dev dependencies) installed in a virtualenv, and are able to develop.\n\nTo add new dependencies, run::\n\n    poetry add <package>\n\nTo activate your virtualenv, run::\n\n    poetry shell\n\nAfterwards, you can run all following commands in the virtualenv. In case you don't, you have to add 'poetry run' before each\nof the next commands to execute it in the virtualenv, or you will get missing requirements errors.\n\nTo run the style checker, run::\n\n    ./scripts/lint.sh\n\nTo run the tests and get some coverage information, run::\n\n    ./scripts/run_tests.sh\n\n.. _poetry: https://python-poetry.org/\n.. _semantic versioning: https://semver.org/\n",
    'author': 'Stefan Kraus',
    'author_email': 'dev@stefankraus.org',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/AliceMurray/tibian',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import abc


class TicketSource(abc.ABC):

    TYPENAME = "unknown"

    def __init__(self, name: str, config: dict, *args, **kwargs) -> None:
        super().__init__()
        self.name = name

    @abc.abstractmethod
    def get_open_tickets(self):
        pass



import datetime

# Some default values to get back, can be changed for default values

_DAY = datetime.date.today() + datetime.timedelta(days=0)
_STD_CONFIG_FILEPATH = "./config.yaml"

# Some functions to get values from default values
# These functions are meant to be mocked in code


def get_today():
    return _DAY


def get_std_config_filepath():
    return _STD_CONFIG_FILEPATH



import datetime
from dataclasses import astuple, dataclass
from typing import TYPE_CHECKING, List

if TYPE_CHECKING:
    from tibian.sources.ticketsource import TicketSource

import tibian.vars as tb_vars
from tibian.targets.target import Target


@dataclass
class Ticket:
    name: str
    title: str
    status: str
    creation_date: datetime.date

    def is_tickets_birthday(self) -> bool:
        day = tb_vars.get_today()
        a = day.replace(year=1) == self.creation_date.replace(year=1) and (
            day != self.creation_date
        )
        return a


@dataclass
class BirthdayTicket(Ticket):
    @property
    def age(self) -> int:
        return tb_vars.get_today().year - self.creation_date.year


def filter_for_birthday_tickets(open_tickets: List[Ticket]) -> List[BirthdayTicket]:
    return [BirthdayTicket(*astuple(t)) for t in open_tickets if t.is_tickets_birthday()]


def collect_open_tickets(sources: List["TicketSource"]):
    open_tickets = []
    for s in sources:
        tickets = s.get_open_tickets()
        open_tickets.extend(tickets)
    return open_tickets


def announce_birthdays(targets: List[Target], birthday_tickets: List[BirthdayTicket]) -> None:
    for target in targets:
        target.announce_birthdays(birthday_tickets)
